//////////////////////////////////////////////////////////////////////////////
//
// PocketHAL
// Copyright 2004 by Thierry Tremblay
//
//////////////////////////////////////////////////////////////////////////////

#ifndef POCKETHAL_FASTCOPY_H
#define POCKETHAL_FASTCOPY_H

#include <PocketHAL/config.h>



//////////////////////////////////////////////////////////////////////////////
//
// FastCopy - Faster then memcpy() for big block of memory (>= 32 bytes)
//
// This is not intended as a replacement for memcpy(), only use for large block
// of memory. memcpy() is usually an intrinsic and can be optimized by the
// compiler depending on the context, this is not the case with FastCopy().
//.
//////////////////////////////////////////////////////////////////////////////

void FastCopy( void* destination, const void* source, unsigned count );




//////////////////////////////////////////////////////////////////////////////
//
// Implementation details
//
//////////////////////////////////////////////////////////////////////////////

/*todo

#if defined(PHAL_CPU_ARM)


    extern "C" void FastCopy_PXA( void* destination, const void* source, unsigned count );

    inline void FastCopy( void* destination, const void* source, unsigned count )
    {
        FastCopy_PXA( destination, source, count );
    }


#elif defined(PHAL_CPU_MIPS)


    extern "C" void FastCopy_MIPS( void* destination, const void* source, unsigned count );

    // We have a MIPS version
    inline void FastCopy( void* destination, const void* source, unsigned count )
    {
        FastCopy_MIPS( destination, source, count );
    }


#else

*/

#if !defined(PHAL_PLATFORM_SYMBIAN)
    #include <memory.h>
#endif

    // Default implementation
    inline void FastCopy( void* destination, const void* source, unsigned count )
    {
        memcpy( destination, source, count );
    }


//#endif



#endif
