//////////////////////////////////////////////////////////////////////////////
//
// PocketHAL
// Copyright 2004 by Thierry Tremblay
//
//////////////////////////////////////////////////////////////////////////////

#ifndef POCKETHAL_RECT_H
#define POCKETHAL_RECT_H



namespace PHAL
{

//////////////////////////////////////////////////////////////////////////////
//
// Point
//
//////////////////////////////////////////////////////////////////////////////

class Point
{
public:
    
    Point()                                     {}
    Point( int ax, int ay )                     { x = ax; y = ay; }

    // Set the point coordinates
    void Set( int ax, int ay )                  { x = ax; y = ay; }
    
    // Translate the point by the specified amount
    void Translate( int dx, int dy )            { x += dx; y += dy; }
    
    // Operators    
    inline Point& operator+=( const Point& p )  { x += p.x; y += p.y; return *this; }
    inline Point& operator-=( const Point& p )  { x -= p.x; y -= p.y; return *this; }

    // Data
    int x;
    int y;
};



inline Point operator+( const Point& a, const Point& b ) { Point p(a); p += b; return p; }
inline Point operator-( const Point& a, const Point& b ) { Point p(a); p -= b; return p; }




//////////////////////////////////////////////////////////////////////////////
//
// Rect
//
//////////////////////////////////////////////////////////////////////////////

class Rect
{
public:
    
    Rect()                                    {}
    Rect( int l, int t, int r, int b)         { left = l; top = t; right = r; bottom = b; }
    Rect( const Point& p, int w, int h )      { left = p.x; top = p.y; right = p.x + w; bottom = p.y + h; }

    // Info
    int   GetWidth() const                    { return right - left; }
    int   GetHeight() const                   { return bottom - top; }
    
    // Is the rectangle empty?
    bool  IsEmpty() const                     { return (right <= left) || (bottom <= top); }
    
    // Check if a point is inside the rectangle
    bool  Contains( const Point& point ) const { return point.x >= left && point.x < right && point.y >= top && point.y < bottom; }

    // Move the rectangle to the specified location
    void  Move( int x, int y )                { Translate( x-left, y-top ); }
    void  Move( const Point& p )              { Translate( p.x-left, p.y-top ); }
    
    // Set the rectangle values
    void  Set( int l, int t, int r, int b )   { left = l; top = t; right = r; bottom = b; }
    
    // Translate the rectangle by the specified amount
    void  Translate( int dx, int dy )         { left += dx; right += dx; top += dy; bottom += dy; }
    void  Translate( const Point& p )         { left += p.x; right += p.x; top += p.y; bottom += p.y; }

    // Data
    int left;
    int top;
    int right;
    int bottom;
};



} // end of namespace PHAL



#endif
