//////////////////////////////////////////////////////////////////////////////
//
// PocketHAL
// Copyright 2004 by Thierry Tremblay
//
//////////////////////////////////////////////////////////////////////////////

#include "SampleUI.h"
#include "SampleView.h"

using PHAL::uint16_t;



//////////////////////////////////////////////////////////////////////////////
//
// SampleUI
//
//////////////////////////////////////////////////////////////////////////////

void SampleUI::ConstructL()
{
    BaseConstructL();

    m_view = new (ELeave) SampleView;
    m_view->SetMopParent(this);
    m_view->ConstructL( ClientRect() );

    AddToStackL( m_view );

    m_idleTimer = CIdle::New( CActive::EPriorityIdle );


    RWindow* window = static_cast<RWindow*>( m_view->DrawableWindow() );

    m_display = PHAL::Display::Create( *window );
    if (!m_display) User::Leave( KErrGeneral );
}



SampleUI::~SampleUI()
{
	if (m_idleTimer)
    {
        m_idleTimer->Cancel();
	    delete m_idleTimer;
    }

    delete m_display;   

    if (m_view)
    {
        RemoveFromStack( m_view );
        delete m_view;
    }
}




void SampleUI::HandleCommandL( TInt aCommand )
{
    switch ( aCommand )
    {
    case EAknSoftkeyExit:
    case EEikCmdExit:
        {
            Exit();
        }
        break;
        
    default:
        break;      
    }
}



void SampleUI::HandleForegroundEventL( TBool aForeground )
{
    Superclass::HandleForegroundEventL( aForeground );

    if (aForeground)
    {
        SetKeyBlockMode( ENoKeyBlock );
        m_idleTimer->Start( TCallBack( IdleTimerCallback, this) );
    }
    else
    {
		m_idleTimer->Cancel();
        SetKeyBlockMode( EDefaultBlockMode );
    }

    m_display->HandleForegroundEvent( aForeground );
}



TKeyResponse SampleUI::HandleKeyEventL( const TKeyEvent&, TEventCode )
{
    return EKeyWasNotConsumed;
}



TInt SampleUI::IdleTimerCallback( TAny* sampleUI )
{
    static_cast<SampleUI*>( sampleUI )->GameLoop();
    return 1;
}



void SampleUI::GameLoop()
{
    if (m_display->BeginScene())
    {
        PHAL::Surface* backbuffer = m_display->GetBackBuffer();
        
        uint16_t color;
        
        if (++m_frame & 1)
        {
            color = 0x1F << 11;
        }
        else
        {
            color = 0x3F << 5;
        }
        
        
        // Draw on backbuffer
        unsigned width  = backbuffer->GetWidth();
        unsigned height = backbuffer->GetHeight();

        for (unsigned y = 0; y != height; ++y)
        {
            uint16_t* pixels = backbuffer->GetPixels( 0, y );

            for (unsigned x = 0; x != width; ++x)
            {
                *pixels++ = color;
            }
        }
        
        // Blit backbuffer to front
        m_display->Swap();
    }
}
